#!/usr/bin/env ruby

# if contents.bom in current directory, upload by sftp to math.columbia.edu
# http://www.rubydoc.info/gems/net-sftp

require 'net/sftp'

server = 'math.columbia.edu'
user = 'bayer'
keys = ["/Users/dave/.ssh/id_rsa"]
local_prefix = '/Local/Web/math.columbia.edu/'
remote_prefix = '/bayer/html/'
bomfile = 'contents.bom'

####################################################

# get file hash, dir list from bomfile
def list_bom(bomfile)
  files = {}
  %x{ lsbom -f -p fcs #{bomfile} }.each_line do |line|
    file, check, size = line.split
    file.sub!('./', '')
    next if file =~ %r{(^|/)[.]}
    files[file] = [check, size]
  end
  dirs = []
  %x{ lsbom -d -s #{bomfile} }.split("\n").each do |dir|
    dir.sub!('./', '')
    next if dir =~ %r{(^|/)[.]}
    dirs << dir   
  end
  [files, dirs]
end

# get dir list from bomfile
def dir_list(bomfile)
  %x{ lsbom -d -s #{bomfile} }.split("\n")
end

# update server
def update(tick, who, &block)
  puts "#{tick} #{who}"
  begin
    yield
  rescue
    puts "    error"
  end
end

####################################################

# check current directory
local_path = Dir.pwd
if not local_path.start_with? local_prefix
  puts 'not a mirror directory for math.columbia.edu'
  exit
elsif not File.file? bomfile
  puts "#{bomfile} not found"
  exit
end
remote_path = local_path.sub local_prefix, remote_prefix

# open sftp session
Net::SFTP.start(server, user, port: 2718, keys: keys) do |sftp|
  puts

  # read bomfiles, mark remote bomfile for replacement
  sftp.download!("#{remote_path}/#{bomfile}", bomfile)
  remote_files, remote_dirs = list_bom(bomfile)
  remote_files[bomfile] = nil
  %x{ mkbom . #{bomfile} }
  local_files, local_dirs = list_bom(bomfile)
  
  # delete remote files not local (except .mp4 videos)
  (remote_files.keys - local_files.keys).each do |file|
    next if file.end_with? '.mp4'
    update('-', file) do
      sftp.remove! "#{remote_path}/#{file}"
    end
  end

  # delete remove dirs not local
  (remote_dirs - local_dirs).reverse.each do |dir|
    update('-', dir) do
      sftp.rmdir! "#{remote_path}/#{dir}"
    end
  end

  # create remote dirs
  (local_dirs - remote_dirs).each do |dir|
    update('+', dir) do
      sftp.mkdir! "#{remote_path}/#{dir}"
    end
  end

  # upload new or changed local files, and command line arguments
  (local_files.keys | ARGV).each do |file|
    if ARGV.include? file
      tick = '!'
    elsif remote_files.key? file and remote_files[file] == local_files[file]
      next
    else
      tick = remote_files.key?(file) ? tick = '^' : tick = '+'
    end
    update(tick, file) do
      sftp.upload!("#{local_path}/#{file}", "#{remote_path}/#{file}")
    end
  end

end
